/**
 * Included in the page context by the sp-list macro. Adds an on-hover effect to each data row in the table.
 */

(function($){

    AJS.SharePoint = AJS.SharePoint || {};

    AJS.SharePoint.SharePointList = {

        init: function(){
            var $tables = $(".sp-document-library");
            $tables.each(function(idx, el){
                $("#" + el.id).treetable({
                    expandable : true,
                    nodeIdAttr : "tt-id",
                    parentIdAttr : "tt-parent-id",
                    branchAttr : "tt-branch",
                    tableId: el.id,
                    onNodeCollapse: function() {
                        var node = this;
                        var $table = $("#" + node.settings.tableId);
                        $table.treetable("unloadBranch", node);
                    },
                    onNodeExpand: function() {
                        var node = this;
                        var $table = $("#" + node.settings.tableId);
                        var requestUri = AJS.SharePoint.SharePointList.buildGetItemsUri(node);
                        // Render loader/spinner while loading
                        var spinnerEl = node.treeCell.children("span.indenter");
                        var spinner = Raphael.spinner(spinnerEl, 10, "#666");
                        $.ajax({
                            async: false, // Must be false, otherwise loadBranch happens after showChildren?
                            url: requestUri
                        }).done(function(jsonData) {

                                var rowsHtml = "";
                                $.each(jsonData.rows, function(i){
                                    var curRow = jsonData.rows[i];
                                    var nodeType = curRow.primaryClass === "document" ? "false" : "true";//true is folder
                                    rowsHtml += "<tr class='" + curRow.primaryClass + "' data-tt-id='" + curRow.id + "' data-tt-parent-id='" + curRow.parentId + "' data-tt-branch='" + nodeType + "' data-item-title='" + curRow.fileRelativePath + "'>";

                                    // the first column is merge column
                                    rowsHtml += "<td>";
                                    $.each(curRow.values, function(j){
                                        var curCol = curRow.values[j];
                                        if(curCol.columnType.type === "Icon"){
                                            rowsHtml += "<img class='sp-icon-align' src='" + curCol.value + "' />";
                                        }else if(curCol.columnType.type === "Document"){
                                            rowsHtml += "<a style='margin-left: 3px' href='" + curCol.link + "'>" + curCol.value + "</a>";
                                        }else if(curCol.columnType.type === "Attachments"){
                                            rowsHtml += "<img title='Attachments' src='" + AJS.params.contextPath + "/download/resources/com.atlassian.confluence.extra.sharepoint/images/attach.gif' />";
                                        }
                                    });
                                    rowsHtml += "</td>";

                                    // next columns
                                    $.each(curRow.values, function(j){
                                        var curCol = curRow.values[j];
                                        if(curCol.columnType.type !== "Icon" && curCol.columnType.type !== "Document" && curCol.columnType.type !== "Attachments") {
                                            rowsHtml += "<td data-column-type='" + curCol.columnType.type + "'>" + curCol.value + "</td>";
                                        }
                                    });
                                    rowsHtml += "</tr>";
                                });

                                $table.treetable("loadBranch", node, rowsHtml);

                                $(".sp-document-library tbody tr").unbind("mousedown").bind("mousedown", function() {
                                    $(".selected").not(this).removeClass("selected");
                                    $(this).toggleClass("selected");
                                });

                                spinner();
                                delete spinner;
                            });
                    }
                });
            });
        },

        setupPropertiesAndEvents: function() {
            // Highlight selected row for library
            $(".sp-document-library tbody tr").bind("mousedown", function() {
                $(".selected").not(this).removeClass("selected");
                $(this).toggleClass("selected");
            });

            //Remove sort event on tree table
            $.each($(".sp-document-library"), function(i){
                var tbl = $(this);
                var iNumberOfColumn = tbl.children("thead").children("tr").children().length;
                var headersSettings = {};
                for(var i = 0; i < iNumberOfColumn; i++)
                {
                    headersSettings[i] = {sorter : false};
                }
                tbl.tablesorter( { headers : headersSettings} );
            });

            // hilight the selected row for sp-list not library
            var rows = $(".sp-data-row");
            if (rows == null || rows == undefined || rows.length == 0) {
                AJS.log("sp-list macro contained no data rows.");
                return;
            }

            rows.mouseover(function() {
                var me = $(this);
                me.addClass("sp-data-row-selected");
                me.find(".sp-actions").addClass("visible");
                me.find(".sp-actions-placeholder").removeClass("visible");
            });

            rows.mouseout(function() {
                var me = $(this);
                me.removeClass("sp-data-row-selected");
                me.find(".sp-actions").removeClass("visible");
                me.find(".sp-actions-placeholder").addClass("visible");
            });
        },

        render: function(){
            this.init();
            this.setupPropertiesAndEvents();
        },

        buildGetItemsUri : function(selectedNode) {
            var $treeTable = selectedNode.row.parents("table");
            var siteAlias = $treeTable.attr("data-site-alias");
            var subSitePath = $treeTable.attr("data-subsite-url");
            var listName = $treeTable.attr("data-list-name");
            var viewName = $treeTable.attr("data-view-name");
            var columns = $treeTable.attr("data-columns");
            var parentId = selectedNode.id;
            var subFolderPath = selectedNode.row.attr("data-item-title");
            if(subSitePath !== undefined && subSitePath != ""){
                var nodeLevel = selectedNode.level();
                var arrFolderPath = subFolderPath.split("/");
                var arrSubSitePath = subSitePath.split("/");
                if(arrFolderPath.length > 0 && arrFolderPath.length > arrSubSitePath.length)
                {
                    // need to trim the prefix
                    subFolderPath = arrFolderPath.slice(arrSubSitePath.length, arrFolderPath.length).join("/");
                }
            }

            var requestUri = AJS.params.contextPath + "/rest/sharepoint/1.0/sp-list/items?siteAlias=" + siteAlias + "&subSitePath="
                + subSitePath + "&listName=" + listName + "&subFolderPath="
                + subFolderPath + "&viewName=" + viewName + "&columns="
                + columns + "&parentId=" + parentId;
            return requestUri;
        }
    };

    AJS.toInit(function($){
        AJS.SharePoint.SharePointList.render();
    });

})(AJS.$);